"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const config_1 = require("./config");
function createWebpackConfig(isServer) {
    const config = getMergedConfig();
    const resolveAppPath = (p) => path.resolve(config.rootDir, p);
    const webpackConfig = {
        bail: true,
        mode: config.isDev ? 'development' : 'production',
        devtool: (config.isDev || isServer) ? 'source-map' : false,
        stats: 'errors-warnings',
        target: isServer ? 'node' : 'web',
        entry: {
            app: isServer ? config.serverEntry : resolveAppPath('src/index.tsx')
        },
        output: {
            path: config.distDir,
            publicPath: isServer ? '/' : '/static',
            filename: isServer ? 'server/[name].js' : 'client/[name].js',
            chunkFilename: isServer ? 'server/[name].chunk.js' : 'client/[name].chunk.js',
            libraryTarget: isServer ? 'commonjs2' : 'umd',
        },
        resolve: {
            extensions: ['.js', '.jsx', '.ts', '.tsx'],
            alias: {
                'server-renderer': isServer
                    ? 'server-renderer/lib/server.js'
                    : 'server-renderer/lib/client.js',
                src: resolveAppPath('src'),
            },
        },
        module: {
            rules: [
                {
                    test: /\.tsx?$/,
                    include: path.resolve(config.rootDir, 'src'),
                    loader: require.resolve('ts-loader'),
                    options: {
                        transpileOnly: true,
                    }
                },
                {
                    test: /\.s?css$/,
                    use: getStyleLoaders(isServer, true, config)
                },
            ]
        }
    };
    return webpackConfig;
}
exports.createWebpackConfig = createWebpackConfig;
const getStyleLoaders = (isServer, isModule, config) => {
    const cssLoaderWithModule = {
        loader: require.resolve('css-loader'),
        options: {
            importLoaders: 1,
            modules: isModule && {
                localIdentName: '[name]-[local]',
            },
            localsConvention: 'camelCaseOnly',
            sourceMap: config.isDev,
        },
    };
    const styleLoader = isServer
        ? require.resolve('isomorphic-style-loader')
        : config.isDev
            ? require.resolve('style-loader')
            : MiniCssExtractPlugin.loader;
    const loaders = [
        styleLoader,
        cssLoaderWithModule,
    ];
    if (!isServer && !config.isDev) {
        loaders.push({
            loader: require.resolve('postcss-loader'),
            options: {
                ident: 'postcss',
                plugins: () => [
                    require('postcss-flexbugs-fixes'),
                    require('postcss-preset-env')({
                        autoprefixer: {
                            flexbox: 'no-2009',
                        },
                        stage: 3,
                    }),
                ],
                sourceMap: false,
            }
        });
    }
    loaders.push(require.resolve('sass-loader'));
    return loaders;
};
// function getTsLoader(isServer: boolean, config: Config): webpack.RuleSetRule {
//   const srcDir = path.resolve(config.rootDir, 'src')
//   if (isServer) {
//     return {
//       test: /\.tsx?$/,
//       include: srcDir,
//       loader: require.resolve('ts-loader'),
//       options: {
//         transpileOnly: true,
//       }
//     }
//   }
//   return {
//     test: /\.tsx?$/,
//     loader: require.resolve('babel-loader'),
//     options: {
//       plugins: [
//         require.resolve('@babel/plugin-proposal-class-properties'),
//         [
//           require.resolve('@babel/plugin-transform-runtime'),
//           {
//             helpers: true,
//             regenerator: true,
//             useESModules: false
//           }
//         ]
//       ],
//       presets: [
//         require.resolve('@babel/preset-react'),
//         require.resolve('@babel/preset-env'),
//         [
//           require.resolve('@babel/preset-typescript'),
//           {
//             isTSX: true,
//             allExtensions: true,
//             allowNamespaces: false,
//           }
//         ],
//       ],
//     }
//   }
// }
function getMergedConfig() {
    const defaultConfig = config_1.getConfig();
    const customConfigPath = path.resolve(defaultConfig.rootDir, 'ssr.config.js');
    if (fs.existsSync(customConfigPath)) {
        const customConfig = require(customConfigPath);
        return Object.assign(Object.assign({}, defaultConfig), customConfig);
    }
    return defaultConfig;
}
